// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>, see README.TXT for details

#include <cassert>
#include <ClanLib/clanlib.h>
#include <ClanLib/clanlayer2.h>

#include "pingu.hh"
#include "actions/pingu_actions.hh"

// Create the static objects of the class
ColMap* Pingu::colmap;
CL_ResourceManager* Pingu::resource;
CL_Surface* Pingu::walker;
CL_Surface* Pingu::faller;

Pingu::Pingu()
{
  action = 0;
  data.x_pos = 50;
  data.y_pos = 50;
  walker_c.set_size(8);
  faller_c.set_size(8);
  cout << "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1111" << endl;
}

Pingu::Pingu(int x, int y)
{
  action = 0;
  data.x_pos = x;
  data.y_pos = y;
  walker_c.set_size(8);
  faller_c.set_size(8);
}

int
Pingu::get_x()
{
  return data.x_pos;
}

int
Pingu::get_y()
{
  return data.y_pos;
}

void 
Pingu::set_pos(int x, int y)
{
  data.x_pos = x;
  data.y_pos = y;
}

void
Pingu::set_colmap(ColMap* a)
{
  colmap = a;
}

void
Pingu::set_map(PinguMap* a)
{
}

void 
Pingu::load_data(CL_ResourceManager* res)
{
  assert(res);
  resource = res;

  walker =  CL_Res_Surface::load("Pingus/walker", *resource);
  faller =  CL_Res_Surface::load("Pingus/faller", *resource);

  assert(walker);
  assert(faller);
}

int
Pingu::set_action(PinguAction* act)
{
  assert(act);

  if ((action && action->can_reset()) || !action) {
    action = act;
    action->set_pingu_data(data);
    action->load_data(resource);
  }
}

bool
Pingu::is_over(int x, int y)
{
  if (data.status == dead || data.status == not_catchable)
    return false;

  if (data.x_pos - 16 < x && data.x_pos + 16 > x &&
      data.y_pos - 32 < y && data.y_pos > y) {
    return true;
  } else {
    return false;
  }
}

void
Pingu::let_move(void)
{
  //assert(action);
  assert(colmap);
  action->set_colmap(colmap); //BUG: the colmap need only to be set once
  
  if (action) { // if it has a action, do it
    action->let_move();
  } else {      // if not, do a walk
    if (rel_getpixel(0, -1) == 0) {
      // falling
      // Bug: Maybe the reason, why the pingus get stick
      ++data.y_pos;
      if (rel_getpixel(0, -1) == 0) 
	++data.y_pos;
      ++data.falling;
    } else {
      data.falling = 0;
      if (rel_getpixel(1, 0) == 0) { // if infront is free
	data.x_pos += data.direction;    
      } else { // if infront is a pixel 
	// Pingu is walking mounten up/down
	if (rel_getpixel(1,1)== 0) {
	  data.x_pos += data.direction;
	  --data.y_pos;
	} else if (rel_getpixel(1,2)== 0) {
	  data.x_pos += data.direction;
	  data.y_pos -= 2;
	} else {
	  change(data.direction);
	}
      }
    }
  }
}

void
Pingu::draw_offset(int x, int y)
{
  if (action)
    action->draw_offset(x, y);
  else {
    if (data.falling > 3)
      faller->put_screen(data.x_pos + x - 16, data.y_pos + y - 32, ++faller_c);
    else
      walker->put_screen(data.x_pos + x - 16, data.y_pos + y - 32, 
			 ++walker_c + ((data.direction == -1) ? 0 : walker_c.size()));
  }
}

int
Pingu::rel_getpixel(int x, int y)
{
  assert(colmap);
  return colmap->getpixel(data.x_pos + (x * data.direction), (data.y_pos) - y);
}

/* EOF */
